import { FormEventHandler, FormHTMLAttributes } from 'react';
import { HttpMethod } from './http-methods';
import { FetchError } from './lib/fetch-error';
export declare type FormSubmission<Data extends Record<string, unknown>> = FormState<Data> & {
    isIdle: boolean;
    isSubmitting: boolean;
    isRouting: boolean;
    isLoading: boolean;
    isSuccess: boolean;
    isError: boolean;
};
export declare function useFormSubmit<Data extends Record<string, unknown>>(name?: string): FormSubmission<Data>;
export declare type FormProps = {
    /**
     *  The method to use for form submissions. `get` appends the form-data to the
     *  URL in name/value pairs, while others send the form-data as an HTTP post
     *  transaction. Defaults to `post`.
     */
    method?: HttpMethod | Uppercase<HttpMethod>;
    /**
     * A callback that's invoked on form submission. Call `event.preventDefault()`
     * if you don't want Form to handle your submission. For example to abort in
     * case of validation errors.
     *
     * @param formData the data that was entered in the form
     */
    onSubmit?: FormEventHandler<HTMLFormElement>;
    /**
     * Called when the form is successfully submitted.
     * @param state The resulting form state
     */
    onSuccess?: (state: FormState) => void;
    /**
     * Called if an error occured during form submission.
     * @param state The resulting form state
     */
    onError?: (state: FormState) => void;
} & Omit<FormHTMLAttributes<HTMLFormElement>, 'onSubmit' | 'onError' | 'method'>;
declare type FormState<Data extends Record<string, unknown> = Record<string, unknown>> = {
    status: 'idle';
    formData?: FormData;
    values?: undefined;
    data?: Data | null;
    error?: FetchError | Error;
} | {
    status: 'submitting';
    formData: FormData;
    values: Record<string, unknown>;
    data?: Data | null;
    error?: FetchError | Error;
} | {
    status: 'routing';
    data: Data | null;
    redirect: {
        from: string;
        to: string;
    };
    formData: FormData;
    values: Record<string, unknown>;
    error?: FetchError | Error;
} | {
    status: 'success';
    data: Data | null;
    formData: FormData;
    values: Record<string, unknown>;
    error?: FetchError | Error;
} | {
    status: 'error';
    data?: Data | null;
    formData: FormData;
    values: Record<string, unknown>;
    error: FetchError | Error;
};
/**
 * Replace your `form` with `Form` to submit it clientside using `fetch`, and get
 * access to the serialized form data in `usePendingSubmit()` to build a great
 * looking loading status.
 */
export declare const Form: import("react").ForwardRefExoticComponent<{
    /**
     *  The method to use for form submissions. `get` appends the form-data to the
     *  URL in name/value pairs, while others send the form-data as an HTTP post
     *  transaction. Defaults to `post`.
     */
    method?: HttpMethod | Uppercase<HttpMethod>;
    /**
     * A callback that's invoked on form submission. Call `event.preventDefault()`
     * if you don't want Form to handle your submission. For example to abort in
     * case of validation errors.
     *
     * @param formData the data that was entered in the form
     */
    onSubmit?: FormEventHandler<HTMLFormElement>;
    /**
     * Called when the form is successfully submitted.
     * @param state The resulting form state
     */
    onSuccess?: (state: FormState) => void;
    /**
     * Called if an error occured during form submission.
     * @param state The resulting form state
     */
    onError?: (state: FormState) => void;
} & Omit<FormHTMLAttributes<HTMLFormElement>, "method" | "onSubmit" | "onError"> & import("react").RefAttributes<HTMLFormElement>>;
export {};
