"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Form = exports.useFormSubmit = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const router_1 = require("next/router");
const react_1 = require("react");
const zustand_1 = __importDefault(require("zustand"));
const fetch_error_1 = require("./lib/fetch-error");
const use_latest_ref_1 = require("./lib/use-latest-ref");
const use_refs_1 = require("./lib/use-refs");
const set_field_1 = require("./runtime/set-field");
const UNNAMED_FORM = Symbol('unnamed-form');
const DEFAULT_STATE = { status: 'idle' };
const useStore = (0, zustand_1.default)((set, get) => ({
    states: {},
    set(name, state) {
        const states = get().states;
        const slice = Object.assign(Object.assign({}, (states[name] || DEFAULT_STATE)), state);
        set({ states: Object.assign(Object.assign({}, states), { [name]: slice }) });
    },
    get(name) {
        return get().states[name] || DEFAULT_STATE;
    },
}));
function useFormSubmit(name) {
    const state = useStore((0, react_1.useCallback)((store) => store.get(name || UNNAMED_FORM), [name]));
    return (0, react_1.useMemo)(() => (Object.assign(Object.assign({}, state), { isIdle: state.status === 'idle', isSubmitting: state.status === 'submitting', isRouting: state.status === 'routing', isLoading: state.status === 'submitting' || state.status === 'routing', isSuccess: state.status === 'success', isError: state.status === 'error' })), [state]);
}
exports.useFormSubmit = useFormSubmit;
function fetchData({ method: methodArg, url: urlArg, data, }) {
    return __awaiter(this, void 0, void 0, function* () {
        // patch must be in all caps: https://github.com/github/fetch/issues/254
        const method = methodArg.toUpperCase();
        const url = new URL(urlArg, window.location.href);
        if (method === 'GET') {
            for (const [field, value] of data.entries()) {
                if (typeof value === 'string') {
                    url.searchParams.set(field, value);
                }
                else {
                    throw new Error('Cannot submit binary form data using GET');
                }
            }
            history.replaceState(null, '', url);
            return fetch(url.toString(), {
                method,
                headers: { accept: 'application/json' },
            });
        }
        return fetch(url.toString(), {
            method,
            headers: { accept: 'application/json' },
            body: data,
        });
    });
}
/**
 * Replace your `form` with `Form` to submit it clientside using `fetch`, and get
 * access to the serialized form data in `usePendingSubmit()` to build a great
 * looking loading status.
 */
exports.Form = (0, react_1.forwardRef)(function Form(_a, forwardRef) {
    var { method = 'post', onSubmit, onSuccess, onError } = _a, props = __rest(_a, ["method", "onSubmit", "onSuccess", "onError"]);
    const router = (0, router_1.useRouter)();
    const ref = (0, use_refs_1.useRefs)(forwardRef);
    const name = props.name || UNNAMED_FORM;
    const set = useStore((0, react_1.useCallback)((store) => store.set, []));
    const state = useStore((0, react_1.useCallback)((store) => store.get(name), [name]));
    // this ref is needed to ensure success/error callbacks
    // aren't called immediately on mount
    const initializedRef = (0, react_1.useRef)(false);
    const onSuccessRef = (0, use_latest_ref_1.useLatestRef)(onSuccess);
    (0, react_1.useEffect)(() => {
        var _a;
        if (!initializedRef.current)
            return;
        if (state.status === 'success') {
            (_a = onSuccessRef.current) === null || _a === void 0 ? void 0 : _a.call(onSuccessRef, state);
        }
    }, [onSuccessRef, state]);
    const onErrorRef = (0, use_latest_ref_1.useLatestRef)(onError);
    (0, react_1.useEffect)(() => {
        var _a;
        if (!initializedRef.current)
            return;
        if (state.status === 'error') {
            (_a = onErrorRef.current) === null || _a === void 0 ? void 0 : _a.call(onErrorRef, state);
        }
    }, [onErrorRef, state]);
    (0, react_1.useEffect)(() => {
        initializedRef.current = true;
    }, [name, set]);
    (0, react_1.useEffect)(() => {
        function transition() {
            return __awaiter(this, void 0, void 0, function* () {
                if (!ref.current)
                    return;
                switch (state.status) {
                    case 'submitting': {
                        try {
                            const response = yield fetchData({
                                url: ref.current.getAttribute('action') ||
                                    location.href.split('?')[0],
                                method: ref.current.getAttribute('method') || 'POST',
                                data: state.formData,
                            });
                            if (response.redirected) {
                                set(name, {
                                    status: 'routing',
                                    data: yield response.json(),
                                    redirect: { from: location.href, to: response.url },
                                    error: undefined,
                                });
                            }
                            else if (response.ok) {
                                set(name, {
                                    status: 'success',
                                    data: yield response.json(),
                                    error: undefined,
                                });
                            }
                            else {
                                set(name, {
                                    status: 'error',
                                    error: yield fetch_error_1.FetchError.create(response),
                                });
                            }
                        }
                        catch (error) {
                            set(name, {
                                status: 'error',
                                error: error,
                            });
                        }
                        break;
                    }
                    case 'routing': {
                        yield router.push(state.redirect.to, undefined, { scroll: true });
                        set(name, {
                            status: 'success',
                        });
                        break;
                    }
                    case 'success': {
                        ref.current.reset();
                        break;
                    }
                }
            });
        }
        void transition();
    }, [state.status]);
    const handleSubmit = (event) => __awaiter(this, void 0, void 0, function* () {
        if (state.status === 'submitting' || state.status === 'routing') {
            event.preventDefault();
            return;
        }
        onSubmit === null || onSubmit === void 0 ? void 0 : onSubmit(event);
        if (event.defaultPrevented)
            return;
        event.preventDefault();
        const formData = new FormData(event.currentTarget);
        const values = {};
        for (const [name, value] of formData.entries()) {
            (0, set_field_1.setField)(values, name, value);
        }
        set(name, {
            status: 'submitting',
            formData,
            values,
        });
    });
    return (0, jsx_runtime_1.jsx)("form", Object.assign({ ref: ref, method: method, onSubmit: handleSubmit }, props), void 0);
});
//# sourceMappingURL=form.js.map